@@ -5,6 +5,8 @@
 # Copyright (C) 2004 Rob Taylor
 # Copyright (C) 2005, 2006 Collabora Ltd. <http://www.collabora.co.uk/>
 #
+# SPDX-License-Identifier: MIT
+#
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
 # files (the "Software"), to deal in the Software without
@@ -33,14 +35,15 @@ import inspect
 from dbus import validate_interface_name, Signature, validate_member_name
 from dbus.lowlevel import SignalMessage
 from dbus.exceptions import DBusException
+from dbus._compat import is_py2
 
 
 def method(dbus_interface, in_signature=None, out_signature=None,
-        async_callbacks=None,
-        sender_keyword=None, path_keyword=None, destination_keyword=None,
-        message_keyword=None, connection_keyword=None,
-        utf8_strings=False, byte_arrays=False,
-        rel_path_keyword=None):
+           async_callbacks=None,
+           sender_keyword=None, path_keyword=None, destination_keyword=None,
+           message_keyword=None, connection_keyword=None,
+           byte_arrays=False,
+           rel_path_keyword=None, **kwargs):
     """Factory for decorators used to mark methods of a `dbus.service.Object`
     to be exported on the D-Bus.
 
@@ -154,7 +157,16 @@ def method(dbus_interface, in_signature=None, out_signature=None,
     validate_interface_name(dbus_interface)
 
     def decorator(func):
-        args = inspect.getargspec(func)[0]
+        if hasattr(inspect, 'Signature'):
+            args = []
+
+            for arg in inspect.signature(func).parameters.values():
+                if arg.kind in (inspect.Parameter.POSITIONAL_ONLY,
+                        inspect.Parameter.POSITIONAL_OR_KEYWORD):
+                    args.append(arg.name)
+        else:
+            args = inspect.getargspec(func)[0]
+
         args.pop(0)
 
         if async_callbacks:
@@ -182,9 +194,9 @@ def method(dbus_interface, in_signature=None, out_signature=None,
             in_sig = tuple(Signature(in_signature))
 
             if len(in_sig) > len(args):
-                raise ValueError, 'input signature is longer than the number of arguments taken'
+                raise ValueError('input signature is longer than the number of arguments taken')
             elif len(in_sig) < len(args):
-                raise ValueError, 'input signature is shorter than the number of arguments taken'
+                raise ValueError('input signature is shorter than the number of arguments taken')
 
         func._dbus_is_method = True
         func._dbus_async_callbacks = async_callbacks
@@ -198,8 +210,12 @@ def method(dbus_interface, in_signature=None, out_signature=None,
         func._dbus_message_keyword = message_keyword
         func._dbus_connection_keyword = connection_keyword
         func._dbus_args = args
-        func._dbus_get_args_options = {'byte_arrays': byte_arrays,
-                                       'utf8_strings': utf8_strings}
+        func._dbus_get_args_options = dict(byte_arrays=byte_arrays)
+        if is_py2:
+            func._dbus_get_args_options['utf8_strings'] = kwargs.get(
+                'utf8_strings', False)
+        elif 'utf8_strings' in kwargs:
+            raise TypeError("unexpected keyword argument 'utf8_strings'")
         return func
 
     return decorator
@@ -325,9 +341,9 @@ def signal(dbus_interface, signature=None, path_keyword=None,
             sig = tuple(Signature(signature))
 
             if len(sig) > len(args):
-                raise ValueError, 'signal signature is longer than the number of arguments provided'
+                raise ValueError('signal signature is longer than the number of arguments provided')
             elif len(sig) < len(args):
-                raise ValueError, 'signal signature is shorter than the number of arguments provided'
+                raise ValueError('signal signature is shorter than the number of arguments provided')
 
         emit_signal.__name__ = func.__name__
         emit_signal.__doc__ = func.__doc__
