@@ -80,6 +80,12 @@ Boolean_tp_new(PyTypeObject *cls, PyObject *args, PyObject *kwargs)
     return self;
 }
 
+static PyObject *
+Boolean_tp_str(PyObject *self)
+{
+    return PyUnicode_FromString(PyObject_IsTrue(self) ? "1" : "0");
+}
+
 static PyObject *
 Boolean_tp_repr(PyObject *self)
 {
@@ -122,7 +128,7 @@ PyTypeObject DBusPyBoolean_Type = {
     0,                                      /* tp_as_mapping */
     0,                                      /* tp_hash */
     0,                                      /* tp_call */
-    0,                                      /* tp_str */
+    Boolean_tp_str,                         /* tp_str */
     0,                                      /* tp_getattro */
     0,                                      /* tp_setattro */
     0,                                      /* tp_as_buffer */
