@@ -2,6 +2,8 @@
 
 # Copyright (C) 2006 Collabora Ltd. <http://www.collabora.co.uk/>
 #
+# SPDX-License-Identifier: MIT
+#
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
 # files (the "Software"), to deal in the Software without
@@ -70,8 +72,6 @@ class Client(SignalTestsImpl):
             logger.error(s)
         logger.info("asking server to Exit")
         Interface(self.obj, INTERFACE_TESTS).Exit(reply_handler=self.quit_reply_handler, error_handler=self.quit_error_handler)
-        # if the server doesn't reply we'll just exit anyway
-        gobject.timeout_add(1000, lambda: (loop.quit(), False)[1])
 
     def quit_reply_handler(self):
         logger.info("server says it will exit")
